@@ -64,6 +64,11 @@ Journalctl_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 
         self->call_dict = PyRun_String("{"
             "'PRIORITY': int,"
+            "'LEADER': int,"
+            "'SESSION_ID': int,"
+            "'USERSPACE_USEC': int,"
+            "'INITRD_USEC': int,"
+            "'KERNEL_USEC': int,"
             "'_UID': int,"
             "'_GID': int,"
             "'_PID': int,"
@@ -75,9 +80,22 @@ Journalctl_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
             "'_SYSTEMD_OWNER_UID': int,"
             "'CODE_LINE': int,"
             "'ERRNO': int,"
+            "'EXIT_STATUS': int,"
             "'_SOURCE_REALTIME_TIMESTAMP': lambda x: datetime.datetime.fromtimestamp(float(x)/1E6),"
             "'__REALTIME_TIMESTAMP': lambda x: datetime.datetime.fromtimestamp(float(x)/1E6),"
+            "'_SOURCE_MONOTONIC_TIMESTAMP': lambda x: datetime.timedelta(microseconds=float(x)),"
             "'__MONOTONIC_TIMESTAMP': lambda x: datetime.timedelta(microseconds=float(x)),"
+#if PY_MAJOR_VERSION >=3
+            "'COREDUMP': bytes,"
+#else
+            "'COREDUMP': str,"
+#endif
+            "'COREDUMP_PID': int,"
+            "'COREDUMP_UID': int,"
+            "'COREDUMP_GID': int,"
+            "'COREDUMP_SESSION': int,"
+            "'COREDUMP_SIGNAL': int,"
+            "'COREDUMP_TIMESTAMP': lambda x: datetime.datetime.fromtimestamp(float(x)/1E6),"
             "}", Py_eval_input, globals, NULL);
     }
 
@@ -137,7 +155,9 @@ Journalctl_init(Journalctl *self, PyObject *args, PyObject *keywds)
     }
 
     int r;
+    Py_BEGIN_ALLOW_THREADS
     r = sd_journal_open(&self->j, flags);
+    Py_END_ALLOW_THREADS
     if (r < 0) {
         PyErr_SetString(PyExc_IOError, "Error opening journal");
         return 1;
@@ -195,13 +215,21 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
 
     int r;
     if (skip == 1LL) {
+        Py_BEGIN_ALLOW_THREADS
         r = sd_journal_next(self->j);
+        Py_END_ALLOW_THREADS
     }else if (skip == -1LL) {
+        Py_BEGIN_ALLOW_THREADS
         r = sd_journal_previous(self->j);
+        Py_END_ALLOW_THREADS
     }else if (skip > 1LL) {
+        Py_BEGIN_ALLOW_THREADS
         r = sd_journal_next_skip(self->j, skip);
+        Py_END_ALLOW_THREADS
     }else if (skip < -1LL) {
+        Py_BEGIN_ALLOW_THREADS
         r = sd_journal_previous_skip(self->j, -skip);
+        Py_END_ALLOW_THREADS
     }else{
         PyErr_SetString(PyExc_ValueError, "Skip number must positive/negative integer");
         return NULL;
@@ -221,7 +249,7 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
     size_t msg_len;
     const char *delim_ptr;
     char bootid[33];
-    PyObject *key, *value;
+    PyObject *key, *value, *cur_value, *tmp_list;
 
     SD_JOURNAL_FOREACH_DATA(self->j, msg, msg_len) {
         delim_ptr = memchr(msg, '=', msg_len);
@@ -233,7 +261,20 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
         if (strncmp(msg, "_BOOT_ID", 8) == 0)
             strncpy(bootid, (char *) delim_ptr + 1, 33);
         value = Journalctl___process_field(self, key, delim_ptr + 1, (const char*) msg + msg_len - (delim_ptr + 1) );
-        PyDict_SetItem(dict, key, value);
+        if (PyDict_Contains(dict, key)) {
+            cur_value = PyDict_GetItem(dict, key);
+            if (PyList_CheckExact(cur_value) && PyList_Size(cur_value) > 1) {
+                PyList_Append(cur_value, value);
+            }else{
+                tmp_list = PyList_New(0);
+                PyList_Append(tmp_list, cur_value);
+                PyList_Append(tmp_list, value);
+                PyDict_SetItem(dict, key, tmp_list);
+                Py_DECREF(tmp_list);
+            }
+        }else{
+            PyDict_SetItem(dict, key, value);
+        }
         Py_DECREF(key);
         Py_DECREF(value);
     }
@@ -282,6 +323,7 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
 #endif
         value = Journalctl___process_field(self, key, cursor, strlen(cursor));
         PyDict_SetItem(dict, key, value);
+        free(cursor);
         Py_DECREF(key);
         Py_DECREF(value);
     }
@@ -431,7 +473,11 @@ Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
 
     PyObject *arg;
     if (whence == SEEK_SET){
-        if (sd_journal_seek_head(self->j) !=0 ) {
+        int r;
+        Py_BEGIN_ALLOW_THREADS
+        r = sd_journal_seek_head(self->j);
+        Py_END_ALLOW_THREADS
+        if (r != 0) {
             PyErr_SetString(PyExc_IOError, "Error seeking to head");
             return NULL;
         }
@@ -445,7 +491,11 @@ Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
         Py_DECREF(Journalctl_get_next(self, arg));
         Py_DECREF(arg);
     }else if (whence == SEEK_END){
-        if (sd_journal_seek_tail(self->j) != 0) {
+        int r;
+        Py_BEGIN_ALLOW_THREADS
+        r = sd_journal_seek_tail(self->j);
+        Py_END_ALLOW_THREADS
+        if (r != 0) {
             PyErr_SetString(PyExc_IOError, "Error seeking to tail");
             return NULL;
         }
@@ -503,7 +553,11 @@ Journalctl_seek_realtime(Journalctl *self, PyObject *args)
         return NULL;
     }
 
-    if (sd_journal_seek_realtime_usec(self->j, timestamp) != 0) {
+    int r;
+    Py_BEGIN_ALLOW_THREADS
+    r = sd_journal_seek_realtime_usec(self->j, timestamp);
+    Py_END_ALLOW_THREADS
+    if (r != 0) {
         PyErr_SetString(PyExc_IOError, "Error seek to time");
         return NULL;
     }
@@ -557,7 +611,11 @@ Journalctl_seek_monotonic(Journalctl *self, PyObject *args)
         }
     }
 
-    if (sd_journal_seek_monotonic_usec(self->j, sd_id, timestamp) != 0) {
+    int r;
+    Py_BEGIN_ALLOW_THREADS
+    r = sd_journal_seek_monotonic_usec(self->j, sd_id, timestamp);
+    Py_END_ALLOW_THREADS
+    if (r != 0) {
         PyErr_SetString(PyExc_IOError, "Error seek to time");
         return NULL;
     }
@@ -582,9 +640,13 @@ Journalctl_wait(Journalctl *self, PyObject *args, PyObject *keywds)
 
     int r;
     if ( timeout == 0LL) {
+        Py_BEGIN_ALLOW_THREADS
         r = sd_journal_wait(self->j, (uint64_t) -1);
+        Py_END_ALLOW_THREADS
     }else{
+        Py_BEGIN_ALLOW_THREADS
         r = sd_journal_wait(self->j, timeout * 1E6);
+        Py_END_ALLOW_THREADS
     }
 #if PY_MAJOR_VERSION >=3
     return PyLong_FromLong(r);
@@ -603,7 +665,11 @@ Journalctl_seek_cursor(Journalctl *self, PyObject *args)
     if (! PyArg_ParseTuple(args, "s", &cursor))
         return NULL;
 
-    if (sd_journal_seek_cursor(self->j, cursor) != 0) {
+    int r;
+    Py_BEGIN_ALLOW_THREADS
+    r = sd_journal_seek_cursor(self->j, cursor);
+    Py_END_ALLOW_THREADS
+    if (r != 0) {
         PyErr_SetString(PyExc_IOError, "Error seeking to cursor");
         return NULL;
     }
@@ -649,7 +715,11 @@ Journalctl_query_unique(Journalctl *self, PyObject *args)
     if (! PyArg_ParseTuple(args, "s", &query))
         return NULL;
 
-    if (sd_journal_query_unique(self->j, query) != 0) {
+    int r;
+    Py_BEGIN_ALLOW_THREADS
+    r = sd_journal_query_unique(self->j, query);
+    Py_END_ALLOW_THREADS
+    if (r != 0) {
         PyErr_SetString(PyExc_IOError, "Error querying journal");
         return NULL;
     }
@@ -883,6 +953,7 @@ initpyjournalctl(void)
 
     Py_INCREF(&JournalctlType);
     PyModule_AddObject(m, "Journalctl", (PyObject *)&JournalctlType);
+    PyModule_AddStringConstant(m, "__version__", "0.5.0");
     PyModule_AddIntConstant(m, "SD_JOURNAL_NOP", SD_JOURNAL_NOP);
     PyModule_AddIntConstant(m, "SD_JOURNAL_APPEND", SD_JOURNAL_APPEND);
     PyModule_AddIntConstant(m, "SD_JOURNAL_INVALIDATE", SD_JOURNAL_INVALIDATE);
